import os
import tempfile
import time
import requests
from .adaptive_formats import FormatStream
from .api import debug, YOUTUBE_PLAYER_HEADERS, YOUTUBE_PLAYER_ENDPOINT, YOUTUBE_CLIENT_VERSION, \
    YOUTUBE_CLIENT_USER_AGENT, YOUTUBE_PLAYER_KEY, get_id, convert_xml_to_srt
from .exeptions import YoutubeAnalyzerExceptions
from .playlists import create_urls, get_videos_playlist, extract_meta_og_title, extract_meta_og_description, \
    extract_meta_og_image


class CaptionsParser:
    def __init__(self, caption_tracks):
        if caption_tracks == 'Not Captions' or caption_tracks is None:
            raise YoutubeAnalyzerExceptions("Not Captions")
        self.__caption_tracks = caption_tracks

    @property
    def get_languages(self) -> list[dict]:
        """Retorna ums lista de dicionários com os idiomas disponíveis"""
        dt = []
        if 'Not Legend' in self.__caption_tracks:
            return [{'caption': 'Not Captions'}]
        else:
            for track in self.__caption_tracks:
                d = {
                    'lang': track['name']['runs'][0]['text'],
                    'code': track['languageCode'],
                    'url': track['baseUrl']
                }
                dt.append(d)
            return dt

    def __get_subtitle_url(self, language_code):
        """Retorna a URL da legenda para o código de idioma especificado."""
        for track in self.__caption_tracks:
            if track['languageCode'] == language_code:
                return track['baseUrl']
        return None

    def download_subtitles(self, language_code, out_dir, logs: bool = None):
        """
        Faz o download das legendas em texto para o idioma especificado.
        :param logs:
        :param language_code: código do idioma
        :param out_dir: diretório para salvar a legenda
        :return:
        """
        if not os.path.exists(out_dir):
            raise TypeError("este diretório não existe!")
        subtitle_url = self.__get_subtitle_url(language_code)
        lang_list = self.get_languages
        output_file_name = 'captions_video'
        for item in lang_list:
            if item['code'] == language_code:
                output_file_name = item.get('lang')

        if not subtitle_url:
            raise YoutubeAnalyzerExceptions(f"Legendas para o idioma '{language_code}' não encontradas.")
        if logs:
            debug('info', "Baixando legenda...", end=' ')
        try:
            response = requests.get(subtitle_url, stream=True)
            response.raise_for_status()  # Verifica por erros HTTP
        except requests.exceptions.RequestException as e:
            raise YoutubeAnalyzerExceptions(f" Erro ao baixar legendas: {e}")
        # Armazenar o conteúdo XML como uma string
        xml_caption = response.text
        if logs:
            time.sleep(1.5)
            debug("true", "\tOK")
            time.sleep(0.4)
        if not output_file_name:
            return xml_caption
        out_file = os.path.join(out_dir, output_file_name)
        # Criar um arquivo temporário e escrever o XML nele
        with tempfile.NamedTemporaryFile(delete=False, suffix='.xml', mode='w', encoding='utf-8') as temp_file:
            temp_file.write(xml_caption)
            temp_file_path = temp_file.name
        if logs:
            debug("info", "convertendo para srt....", end=" ")
            time.sleep(1.5)
        # Converter o XML para o formato SRT
        convert_xml_to_srt(xml_input_file=temp_file_path, srt_file_name=out_file)
        if logs:
            debug("true", "\tOK!")
        os.remove(temp_file_path)
        if logs:
            time.sleep(1.5)
            debug('warn', f"Legendas salvas como: {output_file_name}")


class Captions:
    def __init__(self, url_video: str, hl='en', gl='US', timeZone="America/Los_Angeles"):
        """
        :type url_video: str
        :param url_video: url do video
        :param hl: ex: pt,en
        :param gl: BR,..
        :param timeZone:
        """
        self.__timeZone = timeZone
        self.__gl = gl
        self.__hl = hl
        self.__url = url_video
        self.__data = self.__get_youtube_video_info

    @property
    def __get_youtube_video_info(self):
        url_video = get_id(self.__url)
        video_id = url_video.split("/")[-1]

        headers = YOUTUBE_PLAYER_HEADERS.copy()
        headers["Referer"] = url_video

        params = {
            "token": YOUTUBE_PLAYER_KEY,
            "prettyPrint": "false"
        }

        payload = {
            "context": {
                "client": {
                    "clientName": "ANDROID",
                    "clientVersion": YOUTUBE_CLIENT_VERSION,
                    "androidSdkVersion": 33,
                    "userAgent": YOUTUBE_CLIENT_USER_AGENT,
                    "hl": self.__hl,
                    "gl": self.__gl,
                    "timeZone": self.__timeZone,
                    "utcOffsetMinutes": 0
                }
            },
            "playbackContext": {
                "contentPlaybackContext": {
                    "html5Preference": "HTML5_PREF_WANTS"
                }
            },
            "videoId": video_id
        }

        response = requests.post(YOUTUBE_PLAYER_ENDPOINT, headers=headers, params=params, json=payload)
        response.raise_for_status()
        data = response.json()

        playability_status = data.get("playabilityStatus", {})
        status = playability_status.get("status")
        if status != "OK":
            raise YoutubeAnalyzerExceptions("Não foi possível obter detalhes do vídeo!")
        captions_dict = data.get('captions', {})
        playerCaptionsTracklistRenderer = captions_dict.get('playerCaptionsTracklistRenderer', {})
        captionTracks = playerCaptionsTracklistRenderer.get('captionTracks', 'Not Legend')

        return {
            "captionTracks": captionTracks
        }

    @property
    def captions_in_video(self) -> CaptionsParser:
        """Obtem objeto CaptionsParser com as  Legendas inclusas no vídeo"""
        data = self.__data

        ca = CaptionsParser(data.get('captionTracks', 'Not Captions'))
        return ca



class VideoParser:
    def __init__(self, url: str):
        self.__url = url
        self.__data = self.__get_youtube_video_info

    @property
    def __get_youtube_video_info(self):
        url = get_id(self.__url)
        video_id = url.split("/")[-1]

        headers = YOUTUBE_PLAYER_HEADERS.copy()
        headers["Referer"] = url

        params = {
            "token": YOUTUBE_PLAYER_KEY,
            "prettyPrint": "false"
        }

        payload = {
            "context": {
                "client": {
                    "clientName": "ANDROID",
                    "clientVersion": YOUTUBE_CLIENT_VERSION,
                    "androidSdkVersion": 33,
                    "userAgent": YOUTUBE_CLIENT_USER_AGENT,
                    "hl": "pt",
                    "gl": "BR",
                    "timeZone": "America/Sao_Paulo",
                    "utcOffsetMinutes": 0
                }
            },
            "playbackContext": {
                "contentPlaybackContext": {
                    "html5Preference": "HTML5_PREF_WANTS"
                }
            },
            "videoId": video_id
        }

        response = requests.post(YOUTUBE_PLAYER_ENDPOINT, headers=headers, params=params, json=payload)
        response.raise_for_status()
        data = response.json()

        playability_status = data.get("playabilityStatus", {})
        status = playability_status.get("status")
        if status != "OK":
            raise YoutubeAnalyzerExceptions("Não foi possível obter detalhes do vídeo!")
        streaming_data = data.get("streamingData", {})
        self.__formats = streaming_data.get("formats", [])
        video_details = data.get("videoDetails", {})
        title = video_details.get("title", "")
        detalhes = data['videoDetails']
        shortDescription = detalhes.get('shortDescription', [])
        isLiveContent = detalhes.get("isLiveContent", None)
        isPrivate = detalhes.get("isPrivate", None)
        author = detalhes.get("author", None)
        thumbnail = detalhes.get('thumbnail', {})
        thumbnails = thumbnail.get('thumbnails', [])
        viewCount = detalhes.get("viewCount", 0)

        return {
            "video_id": video_id,
            "title": title,
            "thumbnails": thumbnails,
            "viewCount": viewCount,
            "shortDescription": shortDescription,
            "isLiveContent": isLiveContent,
            "isPrivate": isPrivate,
            'author': author,
            "streams": streaming_data
        }

    @property
    def is_private(self):
        """verifica se o vídeo é privado"""
        data = self.__data
        return data.get('isPrivate', 'Not IsPrivate')

    @property
    def description(self):
        """obter desrcrição"""
        data = self.__data
        return data.get('shortDescription', 'Not Descriptions')

    @property
    def viewCount(self):
        """visualizações"""
        data = self.__data
        return data.get('viewCount', 'Not Views')

    @property
    def thumbnails(self):
        """obter uma lista de urls de thumbs do vídeo"""
        data = self.__data
        return data.get('thumbnails', 'Not thumbnails')

    @property
    def isLiveContent(self):
        """verifiar se é uma live"""
        data = self.__data
        return data.get('isLiveContent', 'Not IsLive')

    @property
    def author(self):
        """obter autor"""
        data = self.__data
        return data.get('author', 'Not Author')

    @property
    def uris_stream(self) -> FormatStream:
        """objeto de streams"""
        data = self.__data
        return FormatStream(streamingData=data.get('streams', 'Not Streams'))

    @property
    def title(self):
        data = self.__data
        return data.get('title', 'Not title')

    @property
    def captions(self) -> Captions:
        """obter objeto captions"""
        return Captions(url_video=self.__url)


class ParserPlaylist:
    def __init__(self, playlist_url: str):
        self.__url_playlist = playlist_url
        self.__data = self.__get_youtube_playlist_info

    @property
    def __get_youtube_playlist_info(self):
        playlist_url = get_videos_playlist(playlist_url=self.__url_playlist)
        if playlist_url is None:
            return {'title': None, 'description': None, 'count': 0, 'image': None,
                    'is_private': True, 'videos': []}
        urls = create_urls(playlist_url)
        videos = []  # Cria uma lista para armazenar todos os vídeos
        title_p = extract_meta_og_title(playlist_url=self.__url_playlist)
        description = extract_meta_og_description(self.__url_playlist)
        image = extract_meta_og_image(playlist_url=self.__url_playlist)
        for e in urls:
            url = e.get('url')
            index = e.get("index")
            thumbnails = e.get('thumbnail')
            title = e.get('title')
            video = {"title": f"{index}.{title}", "url_watch": url, "thumbnails": thumbnails}
            videos.append(video)

        counts = len(videos)
        return {'title': title_p,
                'description': description,
                'count': counts,
                'image': image,
                'is_private': None,
                'videos': videos}

    @property
    def get_all_videos(self):
        """obter todos os vídeos da playlist"""
        dt = self.__data
        return dt.get('videos')

    @property
    def playlist_name(self):
        """obter nome da playlist"""
        dt = self.__data
        return dt.get('title')

    @property
    def description(self):
        """obter descrição da playlist"""
        dt = self.__data
        return dt.get('description')

    @property
    def count(self):
        """obter a quantidade de vídeos da playlist"""
        dt = self.__data
        return dt.get('count')

    @property
    def image(self):
        """obter imagem da playlist"""
        dt = self.__data
        return dt.get('image')

    @property
    def is_private(self):
        """verifica se é privada"""
        dt = self.__data
        return dt.get('is_private')


import json
import requests
from bs4 import BeautifulSoup


def extract_meta_og_title(playlist_url: str):
    """Extrai o conteúdo da meta tag 'og:title'."""
    response = requests.get(playlist_url)
    response.raise_for_status()

    soup = BeautifulSoup(response.text, 'html.parser')

    tag = soup.find('meta', property='og:title')
    if tag:
        return tag['content']
    return None


def extract_meta_og_description(playlist_url: str):
    """Extrai o conteúdo da meta tag 'og:description'."""
    response = requests.get(playlist_url)
    response.raise_for_status()

    soup = BeautifulSoup(response.text, 'html.parser')

    tag = soup.find('meta', property='og:description')
    if tag:
        return tag['content']
    return None


def extract_meta_og_image(playlist_url: str):
    """Extrai o conteúdo da meta tag 'og:image'."""
    response = requests.get(playlist_url)
    response.raise_for_status()

    soup = BeautifulSoup(response.text, 'html.parser')

    tag = soup.find('meta', property='og:image')
    if tag:
        return tag['content']
    return None


def extract_meta_og_image_width(playlist_url: str):
    """Extrai o conteúdo da meta tag 'og:image:width'."""
    response = requests.get(playlist_url)
    response.raise_for_status()

    soup = BeautifulSoup(response.text, 'html.parser')

    tag = soup.find('meta', property='og:image:width')
    if tag:
        return tag['content']
    return None


def extract_meta_og_image_height(playlist_url: str):
    """Extrai o conteúdo da meta tag 'og:image:height'."""
    response = requests.get(playlist_url)
    response.raise_for_status()
    soup = BeautifulSoup(response.text, 'html.parser')
    tag = soup.find('meta', property='og:image:height')
    if tag:
        return tag['content']
    return None


def create_urls(data: dict):
    """
    Cria uma URL de visualização para cada vídeo com base no ID fornecido.

    :param data: Dicionário contendo a lista de vídeos com seus IDs.
    :return: Lista de URLs de visualização dos vídeos.
    """
    url_base = "https://www.youtube.com/watch?v="
    urls = []
    videos = data.get("videos", [])
    for video in videos:
        video_id = video.get('videoId')
        index = video.get("index")
        title = video.get('title')
        thumbnail = video.get('thumbnail')
        if video_id and index:
            url = f"{url_base}{video_id}"
            d = {"index": index, "title": title, "url": url, 'thumbnail': thumbnail}
            urls.append(d)

    return urls


def get_videos_playlist(playlist_url: str):
    """:return ids and indices de videos de uam playlist"""
    for_ids = []
    total = 0
    dts = {"total": total}
    response = requests.get(playlist_url)
    response.raise_for_status()
    soup = BeautifulSoup(response.text, 'html.parser')
    script = soup.find('script', string=lambda t: t and 'var ytInitialData =' in t)
    if script is None:
        raise ValueError('Não foi possível encontrar os dados no html....atualize o script')
    script_content = script.string
    json_text = script_content.split('var ytInitialData =', 1)[1].split(';</script>', 1)[0].strip().replace(';', '')
    data = json.loads(json_text)
    try:
        # Navegue pela estrutura JSON para encontrar a lista de vídeos
        videos = \
            data['contents']['twoColumnBrowseResultsRenderer']['tabs'][0]['tabRenderer']['content'][
                'sectionListRenderer'][
                'contents'][0]['itemSectionRenderer']['contents']
        if not data:
            raise ValueError("playlist privada ou inválida!")
        # Extraia informações
        for video in videos:
            dados = video['playlistVideoListRenderer']
            contents = dados.get("contents")
            for idx, v in enumerate(contents):
                brute = v['playlistVideoRenderer']
                video_id = brute['videoId']
                title = brute['title']['runs'][0]['text']
                thumbnail = brute['thumbnail']['thumbnails']
                highest_res = max(thumbnail, key=lambda x: x['width'])
                if video_id and title:
                    total += 1
                    dts[f'{idx}'] = video_id
                    for_ids.append({'index': idx, 'videoId': video_id, 'title': title, 'thumbnail': highest_res})

        return {'total': total, 'videos': for_ids}
    except KeyError:
        return None





import datetime
import os
import re
from decimal import Decimal
from urllib.parse import urlparse
import colorama
import xml.etree.ElementTree as et
colorama.init(autoreset=True)
YOUTUBE_PLAYER_ENDPOINT = "https://www.youtube.com/youtubei/v1/player"
YOUTUBE_PLAYER_KEY = "AIzaSyAO_FJ2SlqU8Q4STEHLGCilw_Y9_11qcW8"
YOUTUBE_CLIENT_VERSION = "17.43.36"
YOUTUBE_CLIENT_USER_AGENT = f"com.google.android.youtube/{YOUTUBE_CLIENT_VERSION} (Linux; U; Android 13)"
YOUTUBE_PLAYER_HEADERS = {
    "X-Youtube-Client-Name": "3",
    "X-Youtube-Client-Version": YOUTUBE_CLIENT_VERSION,
    "Origin": "https://www.youtube.com",
    "Content-Type": "application/json",
    "User-Agent": YOUTUBE_CLIENT_USER_AGENT
}


def clear():
    if os.name == 'nt':  # Para Windows
        os.system('cls')
    else:  # Para Unix/Linux/Mac
        os.system('clear')


def convert_xml_to_srt(xml_input_file, srt_file_name):
    tree = et.parse(xml_input_file)
    root = tree.getroot()

    with open(srt_file_name, 'w', encoding='utf-8') as srt_file:
        counter = 1
        for p in root.findall('.//p'):
            start_time = int(p.attrib['t'])
            duration = int(p.attrib.get('d', 0))
            end_time = start_time + duration

            # Formatar o tempo no formato SRT (horas:minutos:segundos,milissegundos)
            start_time_srt = format_time1(start_time)
            end_time_srt = format_time1(end_time)

            # Se p.text for None, tentamos pegar o texto das subtags <s>
            if p.text is not None:
                text = re.sub(r'<.*?>', '', p.text).replace("\n", " ")  # Remover tags e substituir quebras de linha
            else:
                # Caso o texto esteja nas subtags <s>, concatenamos o texto de cada subtag
                text = ''.join([s.text if s.text is not None else '' for s in p.findall('.//s')])

            # Formatar o conteúdo como uma legenda SRT
            srt_file.write(f"{counter}\n")
            srt_file.write(f"{start_time_srt} --> {end_time_srt}\n")
            srt_file.write(f"{text.strip()}\n\n")
            counter += 1


def format_time1(time_ms):
    """Converte o tempo de milissegundos para o formato SRT"""
    hours = time_ms // (1000 * 60 * 60)
    minutes = (time_ms // (1000 * 60)) % 60
    seconds = (time_ms // 1000) % 60
    milliseconds = time_ms % 1000
    return f"{hours:02}:{minutes:02}:{seconds:02},{milliseconds:03}"


def format_time(seconds):
    """Função para formatar o tempo em horas, minutos e segundos corretamente."""
    hours = int(seconds // 3600)
    minutes = int((seconds % 3600) // 60)
    seconds = int(seconds % 60)
    return f"{hours:02}:{minutes:02}:{seconds:02}"


def is_valid(url: str) -> bool:
    """
    Verifica se a URL fornecida é válida e pertence ao YouTube (vídeo ou playlist).

    Args:
        url (str): A URL a ser verificada.

    Returns:
        bool: True se a URL for válida e pertencer ao YouTube, False caso contrário.
    """
    if not url.startswith("https://"):
        return False

    # Verifica se o domínio é do YouTube
    parsed_url = urlparse(url)
    if parsed_url.netloc not in ["www.youtube.com", "youtube.com", "m.youtube.com", "youtu.be"]:
        return False

    # Expressão regular para verificar IDs de vídeos e playlists
    regex = (
        r'(?:youtube\.com\/(?:.*?\/)?(?:playlist\?list=|watch\?v=|e(?:mbed)?\/|shorts\/|live\/)?|youtu\.be\/)'
        r'([a-zA-Z0-9_-]{11})'  # ID de vídeo
        r'|playlist\?list=([a-zA-Z0-9_-]+)'  # ID de playlist
    )

    return bool(re.search(regex, url))


def get_id(url: str) -> str:
    """
    Obtém o ID do vídeo do YouTube a partir de uma URL.

    :param url: URL do vídeo do YouTube.
    :return: ID do vídeo do YouTube, ou None se não for possível extrair o ID.
    """
    regex = (
        r'(?:youtube\.com\/(?:[^\/\n\s]+\/\S+\/|(?:v|e(?:mbed)?)\/|\S*?[?&]v=|live\/|shorts\/)?|youtu\.be\/)'
        r'([a-zA-Z0-9_-]{11})'
    )
    match = re.search(regex, url)
    if match:
        video_id = match.group(1)
        if 'live/' in url:
            return f"https://www.youtube.com/live/{video_id}"
        elif 'shorts/' in url:
            return f"https://www.youtube.com/shorts/{video_id}"
        else:
            return f"https://www.youtube.com/embed/{video_id}"
    return ''


def get_id_playlists(url: str) -> str:
    """
    :param url: url da playlist
    :return: id
    """
    regex = r'[?&]list=([a-zA-Z0-9_-]+)'
    match = re.search(regex, url)
    if match:
        return match.group(1)
    return ''


def debug(type_d: str, msg, end='\n'):
    """Exibe mensagens de depuração com cores baseadas no tipo.

    Args:
        type_d (str): Tipo da mensagem (erro, info, warn).
        msg : Mensagem a ser exibida.
        end (str, optional): String a ser impressa após a mensagem. Padrão é '\n'.
    """
    colors = {
        'erro': colorama.Fore.LIGHTRED_EX,
        'info': colorama.Fore.LIGHTCYAN_EX,
        'warn': colorama.Fore.LIGHTYELLOW_EX,
        'true': colorama.Fore.LIGHTGREEN_EX
    }

    # Obter a cor com base no tipo
    color = colors.get(type_d, colorama.Fore.RESET)

    # Imprimir a mensagem com a cor especificada
    print(f'{color} {msg}', end=end)


def format_bytes(size):
    """Formata bytes em KB, MB, GB, etc."""
    try:
        size = int(size)
        for unit in ['B', 'KB', 'MB', 'GB', 'TB']:
            if int(size) < 1024:
                return f"{size:.2f} {unit}"
            size /= 1024
        return f"{int(size):.2f} PB"
    except Exception:
        return size


def timestamp(timestamps):
    """

    :type timestamps: numero
    """
    try:
        seconds = int(timestamps) / 1_000_000
        # Converta para datetime
        dt = datetime.datetime.fromtimestamp(seconds)
        return dt.strftime('%Y-%m-%d %H:%M:%S')
    except Exception:
        return timestamps


def ms_convert(ms):
    try:
        # Converte milissegundos para segundos
        segundos = int(ms) // 1000
        # Calcula horas, minutos e segundos
        horas = segundos // 3600
        minutos = (segundos % 3600) // 60
        segundos_restantes = segundos % 60

        # Retorna a string formatada
        return f'{horas:02}:{minutos:02}:{segundos_restantes:02}'
    except Exception:
        return ms


def mon_ste(can: int):
    if can == 1:
        return "Mono"
    elif can == 2:
        return "Stéreo"
    elif can is None:
        return "Not audio"
    else:
        return can


def convert_bitrate_precise(bitrate_bps):
    """
    Converte a taxa de bits de bits por segundo (bps) para Kbps e Mbps usando precisão exata com Decimal.

    :param bitrate_bps: Taxa de bits em bps (bits por segundo).
    """
    try:
        # Usando Decimal para maior precisão
        bitrate_kbps = Decimal(int(bitrate_bps)) / Decimal(1000)
        return f"{bitrate_kbps}kbps"
    except Exception:
        return bitrate_bps




import os
import sys
import time
from typing import Optional, Callable
import requests
from .api import debug, format_bytes, timestamp, ms_convert, mon_ste, convert_bitrate_precise
from .exeptions import YoutubeAnalyzerExceptions


class VideoStream:
    def __init__(self, uri):
        self.__stream = self.__load_uri(uri=uri)

    def __load_uri(self, uri):
        self.__data = {'itag': uri.get('itag', 'Not itag'),
                       'url': uri.get('url', 'Not Url'),
                       'mimeType': uri.get('mimeType', 'Not MimeType'),
                       'bitrate': uri.get('bitrate', 'Not Bitrate'),
                       'width': uri.get('width', 'Not Width'),
                       'height': uri.get('height', 'Not Height'),
                       'initRange': uri.get('initRange', 'Not InitRange'),
                       'indexRange': uri.get('indexRange', 'Not indexRange'),
                       'lastModified': uri.get('lastModified', 'Not lastModified'),
                       'contentLength': uri.get('contentLength', 'Not contentLength'),
                       'quality': uri.get('quality', 'Not quality'),
                       'fps': uri.get('fps', 'Not fps'),
                       'qualityLabel': uri.get('qualityLabel', 'Not qualityLabel'),
                       'projectionType': uri.get('projectionType', 'Not projectionType'),
                       'averageBitrate': uri.get('averageBitrate', 'Not averageBitrate'),
                       'approxDurationMs': uri.get('approxDurationMs', 'Not approxDurationMs'),
                       'audioQuality': uri.get('audioQuality', 'Not audio'),
                       'audioSampleRate': uri.get('audioSampleRate', 'Not audioSampleRate'),
                       'audioChannels': uri.get("audioChannels", 'Not audioChannels')
                       }
        return self.__data

    @property
    def itag(self):
        return self.__data.get('itag')

    @property
    def url(self):
        return self.__data.get('url')

    @property
    def mimeType(self):
        return self.__data.get('mimeType')

    @property
    def bitrate(self):
        return convert_bitrate_precise(self.__data.get('bitrate'))

    @property
    def width(self):
        return self.__data.get('width')

    @property
    def height(self):
        return self.__data.get('height')

    @property
    def initRange(self):
        return self.__data.get('initRange')

    @property
    def indexRange(self):
        return self.__data.get('indexRange')

    @property
    def lastModified(self):
        return timestamp(self.__data.get('lastModified'))

    @property
    def contentLength(self):
        return format_bytes(self.__data.get('contentLength'))

    @property
    def quality(self):
        return self.__data.get('quality')

    @property
    def fps(self):
        return self.__data.get('fps')

    @property
    def qualityLabel(self):
        return self.__data.get('qualityLabel')

    @property
    def projectionType(self):
        return self.__data.get('projectionType')

    @property
    def averageBitrate(self):
        return self.__data.get('averageBitrate')

    @property
    def approxDurationMs(self):
        return ms_convert(self.__data.get('approxDurationMs'))

    @property
    def audioQuality(self):
        return self.__data.get('audioQuality')

    @property
    def audioSampleRate(self):
        return self.__data.get('audioSampleRate')

    @property
    def audioChannels(self):
        return mon_ste(self.__data.get('audioChannels'))

    def download_video(self, title: str,
                       output_dir: str = 'youtube_analyzer_downloads',
                       overwrite_output: bool = False,
                       logs: bool = None,
                       capture_chunks: Optional[Callable[[int], None]] = None):
        """
        Faz o download de um vídeo e salva no diretório especificado. Se `logs` for fornecido, exibirá uma barra de
        progresso; caso contrário, retornará os chuncks baixados que deve ser iterado por um loop para baixar :param
        title: Título do vídeo. :param output_dir: Diretório de saída onde o vídeo será salvo. :param
        overwrite_output: Se True, sobrescreve o arquivo se ele já existir. :param logs: Função de callback para
        logs, exibira todo progresso :param capture_chunks: Função de callback para capturar o andamento do download.
        :return:
        """
        # Cria o diretório de saída se não existir
        if not os.path.exists(output_dir):
            if logs:
                debug("warn", f"O diretório {output_dir},não existe criando", end=' ')
            os.makedirs(output_dir, exist_ok=True)
            if logs:
                debug('true', '\t Criado!')
        # Define o caminho completo para salvar o arquivo
        file_path = os.path.join(output_dir, f"{title}.mp4")
        # Verifica se o arquivo já existe
        file_exists = os.path.exists(file_path)
        # Sobrescrever o arquivo se `overwrite_output` for True
        if file_exists and overwrite_output:
            os.remove(file_path)
            file_exists = False

        if file_exists:
            raise FileExistsError(f"Arquivo já existe: {file_path}")
        uri = self.url
        debug('true', msg='Informações Do Vídeo')
        f"""
        {debug('info', 'Fps:', end=' ')}
        {debug('true', self.fps)}
        {debug('info', 'Duração:', end=' ')}
        {debug('true', self.approxDurationMs)}
        {debug('info', 'MimeType:', end=' ')}
        {debug('true', self.mimeType.split(";")[0])}
        {debug('info', 'Data de Modificação:', end=' ')}
        {debug('true', self.lastModified)}
        {debug('info', 'Codecs:', end=' ')}
        {debug('true', self.mimeType.split(";")[1].split("codecs=")[1].replace('"', ''))}
        {debug('info', 'Resolução:', end=' ')}
        {debug('true', f'{self.width}x{self.height}')}
        {debug('info', 'Canais de Áudio:', end=' ')}
        {debug('true', f'{self.audioChannels}')}
        {debug('info', 'Qualidade:', end=' ')}
        {debug('true', f'{self.qualityLabel.lower()}')}
        {debug('info', 'Tamanho do Arquivo:', end=' ')}
        {debug('true', f'{self.contentLength}')}
        """
        time.sleep(3)

        def download_generator():
            """Gerador que baixa o vídeo em pedaços e atualiza o progresso detalhado."""
            try:
                chunk_size = 32768  # 32 KB, padrão do FFmpeg

                response = requests.get(uri, stream=True)
                response.raise_for_status()  # Levanta exceção para códigos de status HTTP de erro

                total_length = int(response.headers.get('content-length', 0))
                with open(file_path, 'wb') as out_file:
                    downloaded = 0
                    for chunk in response.iter_content(chunk_size=chunk_size):
                        if chunk:
                            out_file.write(chunk)
                            downloaded += len(chunk)
                            if capture_chunks:
                                capture_chunks(len(chunk))
                            if logs:
                                percentage = (downloaded / total_length) * 100
                                sys.stdout.flush()
                                sys.stdout.write(
                                    f"\rProgresso: {percentage:.1f}%")
                                sys.stdout.flush()
                            yield len(chunk)
                return file_path
            except requests.exceptions.RequestException as e:
                raise ConnectionError(f"Erro ao baixar o vídeo: {e}")

        if logs:
            for _ in download_generator():
                pass
            if logs:
                debug("true", f"\tDownload completo!")
            return file_path
        else:
            # Retorna o gerador para uso posterior
            return download_generator()


class AudioStream:
    def __init__(self, uri: dict):
        self.__stream = self.__load_uri(uri=uri)

    def __load_uri(self, uri):
        self.__data = {
            'itag': uri.get('itag', 'Not itag'),
            'url': uri.get('url', 'Not url'),
            'mimeType': uri.get('mimeType', 'Not mimeType'),
            'bitrate': uri.get('bitrate', 'Not bitrate'),
            'initRange': uri.get('initRange', 'Not initRange'),
            'indexRange': uri.get('indexRange', 'Not indexRange'),
            'lastModified': uri.get('lastModified', 0),
            'contentLength': uri.get('contentLength', 0),
            'quality': uri.get('quality', 'Not quality'),
            'projectionType': uri.get('projectionType', 'Not projectionType'),
            'averageBitrate': uri.get('averageBitrate', 'Not averageBitrate'),
            'highReplication': uri.get('highReplication', 'Not highReplication'),
            'audioQuality': uri.get('audioQuality', 'Not audioQuality'),
            'approxDurationMs': uri.get('approxDurationMs', 'Not approxDurationMs'),
            'audioSampleRate': uri.get('audioSampleRate', 'Not audioSampleRate'),
            'audioChannels': uri.get('audioChannels', 'Not audioChannels'),
            'loudnessDb': uri.get('loudnessDb', 'Not loudnessDb')
        }
        return self.__data

    @property
    def itag(self):
        return self.__data.get('itag')

    @property
    def url(self):
        return self.__data.get('url')

    @property
    def mimeType(self):
        return self.__data.get('mimeType')

    @property
    def bitrate(self):
        return convert_bitrate_precise(self.__data.get('bitrate'))

    @property
    def initRange(self):
        return self.__data.get('initRange')

    @property
    def indexRange(self):
        return self.__data.get('indexRange')

    @property
    def lastModified(self):
        return timestamp(self.__data.get('lastModified'))

    @property
    def contentLength(self):
        return format_bytes(self.__data.get('contentLength'))

    @property
    def quality(self):
        return self.__data.get('quality')

    @property
    def projectionType(self):
        return self.__data.get('projectionType')

    @property
    def averageBitrate(self):
        return self.__data.get('averageBitrate')

    @property
    def highReplication(self):
        return self.__data.get('highReplication')

    @property
    def audioQuality(self):
        return self.__data.get('audioQuality')

    @property
    def approxDurationMs(self):
        return ms_convert(self.__data.get('approxDurationMs'))

    @property
    def audioSampleRate(self):
        return self.__data.get('audioSampleRate')

    @property
    def audioChannels(self):
        return mon_ste(self.__data.get('audioChannels'))

    @property
    def loudnessDb(self):
        return self.__data.get('loudnessDb')

    def download_audio(self, title: str,
                       output_dir: str = 'youtube_analyzer_downloads',
                       overwrite_output: bool = False,
                       logs: bool = None,
                       capture_chunks: Optional[Callable[[int], None]] = None):
        """
        Faz o download de um vídeo e salva no diretório especificado. Se `logs` for fornecido, exibirá uma barra de
        progresso; caso contrário, retornará os chuncks baixados que deve ser iterado por um loop para baixar :param
        title: Título do vídeo. :param output_dir: Diretório de saída onde o vídeo será salvo. :param
        overwrite_output: Se True, sobrescreve o arquivo se ele já existir. :param logs: Função de callback para
        logs, exibira todo progresso :param capture_chunks: Função de callback para capturar o andamento do download.
        :return:
        """
        # Cria o diretório de saída se não existir
        if not os.path.exists(output_dir):
            if logs:
                debug("warn", f"O diretório {output_dir},não existe criando", end=' ')
            os.makedirs(output_dir)
            if logs:
                debug('true', '\t Criado!')
        # Define o caminho completo para salvar o arquivo
        file_path = os.path.join(output_dir, f"{title}.mp4")
        # Verifica se o arquivo já existe
        file_exists = os.path.exists(file_path)
        # Sobrescrever o arquivo se `overwrite_output` for True
        if file_exists and overwrite_output:
            os.remove(file_path)
            file_exists = False

        if file_exists:
            raise FileExistsError(f"Arquivo já existe: {file_path}")
        uri = self.url
        debug('true', msg='Informações Do Áudio')
        f"""
        {debug('info', 'Bitrate (Taxa de Bits):', end=' ')}
        {debug('true', self.bitrate)}
        {debug('info', 'Duração:', end=' ')}
        {debug('true', self.approxDurationMs)}
        {debug('info', 'MimeType:', end=' ')}
        {debug('true', self.mimeType.split(";")[0])}
        {debug('info', 'Data de Modificação:', end=' ')}
        {debug('true', self.lastModified)}
        {debug('info', 'Codecs:', end=' ')}
        {debug('true', self.mimeType.split(";")[1].split("codecs=")[1].replace('"', ''))}
        {debug('info', 'Canais de Áudio:', end=' ')}
        {debug('true', f'{self.audioChannels}')}
        {debug('info', 'Qualidade:', end=' ')}
        {debug('true', f'{self.audioQuality.lower()}')}
        {debug('info', 'Tamanho do Arquivo:', end=' ')}
        {debug('true', f'{self.contentLength}')}
        """
        time.sleep(3)

        def download_generator():
            """Gerador que baixa o vídeo em pedaços e atualiza o progresso detalhado."""
            try:
                chunk_size = 32768  # 32 KB, padrão do FFmpeg

                response = requests.get(uri, stream=True)
                response.raise_for_status()  # Levanta exceção para códigos de status HTTP de erro

                total_length = int(response.headers.get('content-length', 0))
                with open(file_path, 'wb') as out_file:
                    downloaded = 0
                    for chunk in response.iter_content(chunk_size=chunk_size):
                        if chunk:
                            out_file.write(chunk)
                            downloaded += len(chunk)
                            if capture_chunks:
                                capture_chunks(len(chunk))
                            if logs:
                                percentage = (downloaded / total_length) * 100
                                sys.stdout.flush()
                                sys.stdout.write(
                                    f"\rProgresso: {percentage:.1f}%")
                                sys.stdout.flush()
                            yield len(chunk)
                return file_path
            except requests.exceptions.RequestException as e:
                raise ConnectionError(f"Erro ao baixar o vídeo: {e}")

        if logs:
            for _ in download_generator():
                pass
            if logs:
                debug("true", f"\tDownload completo!")
            return file_path
        else:
            # Retorna o gerador para uso posterior
            return download_generator()


class FormatStream:
    def __init__(self, streamingData):
        self.__data = streamingData
        self.__adaptiveFormats = self.__data.get('adaptiveFormats', [])
        self.__formats = self.__data.get('formats', [])

        self.__videos_streams_data = self.__load_videos_streams
        self.__audios_streams_data = self.__load_audio_streams

        if not self.__adaptiveFormats:
            raise YoutubeAnalyzerExceptions("Not adaptiveFormats")

    def __load_streams_by_mime(self, mime_types: list[str]) -> list[dict]:
        formats = []
        add = []
        for i in self.__adaptiveFormats + self.__formats:
            mimeType = i.get("mimeType", None)
            if any(mime in mimeType for mime in mime_types):
                if not i in add:
                    formats.append(i)
                    add.append(i)
        return formats

    @property
    def __load_videos_streams(self):
        return self.__load_streams_by_mime(['video/mp4', 'video/webm'])

    @property
    def __load_audio_streams(self):
        return self.__load_streams_by_mime(['audio/mp4', 'audio/webm'])

    @property
    def get_resolutions(self) -> list[dict]:
        """
        obter todas resoluções disponíveis,tenha atenção pois como políticas do youtube apenas resolução de até 720p
        tem audio embutido as demais a faixa de áudio é separada.

        :return: uma lista com as resoluçoes e seus mimeType
        """
        resolutions = []
        mimeTypes_add = []
        for uri in self.__videos_streams_data:
            width = uri.get("width", None)
            height = uri.get("height", None)
            mimeType = uri.get("mimeType", None)
            if width and height and mimeType:
                resolution = f'{width}x{height}'
                dt = {"resolution": resolution, "typeUri": str(mimeType).split(';')[0]}
                if not dt in mimeTypes_add:
                    resolutions.append(dt)
                    mimeTypes_add.append(dt)
        return resolutions

    def filter_resolution(self, resolution_filter: str, typeuri: str) -> VideoStream:
        """
        filtre e obtenha apenas uma uri da stream:

        :param resolution_filter: EX: 1920x1080
        :param typeuri: EX: video/mp4
        :return: objeto VideoStream
        """
        # Validate and parse the resolution_filter
        try:
            width, height = map(int, resolution_filter.split('x'))
        except ValueError:
            raise YoutubeAnalyzerExceptions("Invalid resolution format. Expected format 'WIDTHxHEIGHT'.")

        # Iterate through the video streams to find a matching one
        for stream_data in self.__videos_streams_data:
            stream_width = int(stream_data.get('width', 0))
            stream_height = int(stream_data.get('height', 0))
            mime_type = stream_data.get('mimeType', None)

            # Check if both resolution and mime type match the criteria
            if stream_width == width and stream_height == height and typeuri in str(mime_type).split(';')[0]:
                return VideoStream(uri=stream_data)

    @property
    def get_all_audios_quality(self) -> list[dict]:
        """obtenha todas qualidades de áudio disponivel na stream"""
        audios = []
        add = []
        for uri in self.__audios_streams_data:
            quality = uri.get("audioQuality", None)
            mimeType = uri.get("mimeType", None)
            if quality and mimeType:
                dt = {"audioQuality": quality, "typeAudio": str(mimeType).split(';')[0]}

                if not dt in add:
                    audios.append(dt)
                    add.append(dt)
        return audios

    def filter_audio_quality(self, audio_quality: str, type_audio: str) -> AudioStream:
        """
        filtre e obtenha apenas uma faixa de aúdio especifica da stream,

        :param audio_quality: Ex: 'AUDIO_QUALITY_MEDIUM'
        :param type_audio: Ex: 'audio/mp4'
        :return: Objeto AudioStream
        """
        # Iterate through the video streams to find matching ones
        for stream_data in self.__audios_streams_data:
            audioQuality = stream_data.get('audioQuality', '')
            mime_type = stream_data.get('mimeType', '')
            # Check if both resolution and mime type match the criteria
            if audioQuality == audio_quality and type_audio in str(mime_type).split(';')[0]:
                s = AudioStream(uri=stream_data)
                return s

    @property
    def get_highest_resolution(self) -> VideoStream:
        """
        Encontra a maior resolução a partir de uma lista de resoluções disponíveis no vídeo,elas geralmente
        não terão o áudio embutido
   :return (objeto) VideoStream
        """
        resolutions = self.get_resolutions
        if not resolutions:
            raise YoutubeAnalyzerExceptions("Not resolutions in video...")

        # Encontra a maior resolução
        highest_resolution = max(resolutions,
                                 key=lambda r: (int(r['resolution'].split('x')[0]), int(r['resolution'].split('x')[1])))
        r = self.filter_resolution(resolution_filter=highest_resolution.get('resolution'),
                                   typeuri=highest_resolution.get('typeUri'))
        return r

    @property
    def get_best_audio_quality(self) -> AudioStream:
        """
        Obtém a melhor faixa de áudio

        :return(dict): chaves -> 'audioQuality', 'typeAudio'
        """
        # Definir uma ordem de prioridade para a qualidade do áudio
        quality_order = {
            'AUDIO_QUALITY_LOW': 1,
            'AUDIO_QUALITY_MEDIUM': 2,
            'AUDIO_QUALITY_HIGH': 3
        }
        best_quality = None
        best_audio = None
        audio_list = self.get_all_audios_quality

        for audio in audio_list:
            quality = audio.get('audioQuality')
            if quality and (best_quality is None or quality_order.get(quality, 0) > quality_order.get(best_quality, 0)):
                best_quality = quality
                best_audio = audio
        d = self.filter_audio_quality(audio_quality=best_audio.get('audioQuality'),
                                      type_audio=best_audio.get('typeAudio'))

        return d

    @property
    def get_format_contained_audio(self) -> VideoStream:
        """Objeto VideoStream que contém áudio embutido geralmente terá uma baixa resolução"""
        f = max(self.__formats, key=lambda x: x['width'])
        return VideoStream(f)




class YoutubeAnalyzerExceptions(Exception):
    def __init__(self, message: str):
        super().__init__(message)

    def __str__(self):
        """
        Retorna a representação em string da exceção.

        Returns:
            str: Mensagem de erro formatada com detalhes adicionais, se presentes.
        """

        return super().__str__()



import os
import re
import subprocess
import sys
import tempfile
import time
import argparse
from youtube_analyzer import VideoMetadates, PlaylistMetadates
from .api import clear

try:
    from ffmpeg_for_python import FFmpeg
except ImportError:
    subprocess.check_call([sys.executable, "-m", "pip", "install", "ffmpeg_for_python"])
    from ffmpeg_for_python import FFmpeg
try:
    import emoji
except ImportError:
    subprocess.check_call([sys.executable, "-m", "pip", "install", "emoji"])
    import emoji
clear()
time.sleep(2)


class Playlists:

    @staticmethod
    def sanitize_filename(filename: str) -> str:
        r"""Remove caracteres inválidos no Windows: \ / : * ? " < > | e emojis"""
        sanitized = re.sub(r'[\\/*?:"<>|]', "", filename)
        sanitized = emoji.replace_emoji(sanitized, replace='')  # Remove qualquer emoji
        return sanitized

    @staticmethod
    def get_info_video(url_video):
        try:
            v = VideoMetadates()
            video_info = v.get_video_info(url_video=url_video)
            resolution = video_info.uris_stream.get_highest_resolution

            return resolution
        except Exception as e:
            print(f"Erro ao obter informações de vídeo: {e}")

    @staticmethod
    def get_info_audio(url_video):
        try:
            v = VideoMetadates()
            audio_info = v.get_video_info(url_video=url_video)
            audioQuality = audio_info.uris_stream.get_best_audio_quality
            return audioQuality
        except Exception as e:
            print(f"Erro ao obter informações de audio: {e}")

    @staticmethod
    def remux(a_path, v_path, out):
        try:
            ffmpeg = FFmpeg()
            ffmpeg.reset_ffmpeg()
            (ffmpeg
             .overwrite_output
             .input(a_path)
             .input(v_path)
             .output(out)
             .copy_codecs
             .run(capture_output=True))
        except Exception as e:
            print(f"Erro ao remuxar: {e}")

    def download_playlist_videos(self, playlist_url, output_dir, audio=None):
        os.makedirs(output_dir, exist_ok=True)
        temp_dir = tempfile.mkdtemp('downloads_yt_')
        pl = PlaylistMetadates()
        playlist_info = pl.get_playlist_info(playlist_url=playlist_url)

        if playlist_info.is_private:
            print("A playlist é privada. Não é possível baixar vídeos.")
            return
        videos = playlist_info.get_all_videos
        print(f"Encontrados {len(videos)} vídeos na playlist.")

        for video in videos:
            video_url = video.get('url_watch')
            video_title = video.get('title')
            if audio:
                uri_a = self.get_info_audio(url_video=video_url)
                title_a = self.sanitize_filename(f"AUDIO_{video_title}")
                print(f"Baixando Audio: {video_title}")
                uri_a.download_audio(title=title_a, output_dir=temp_dir, overwrite_output=True, logs=True)

            print(f"Baixando Vídeo: {video_title}")
            uri = self.get_info_video(video_url)
            title_v = self.sanitize_filename(video_title)
            out = os.path.join(output_dir, f"{title_v}.mp4")
            v_path = uri.download_video(title=title_v, overwrite_output=True, output_dir=temp_dir, logs=True)
            uri_a = self.get_info_audio(url_video=video_url)
            title_a = self.sanitize_filename(f"AUDIO_{video_title}")
            print(f"Baixando Audio: {video_title}")
            a_path = uri_a.download_audio(title=title_a, output_dir=temp_dir, overwrite_output=True, logs=True)
            print("Remuxando.........")
            time.sleep(2)
            self.remux(a_path=a_path, v_path=v_path, out=out)
            os.remove(a_path)
            os.remove(v_path)

    def download_video(self, video_url, output_dir):
        os.makedirs(output_dir, exist_ok=True)
        temp_dir = tempfile.mkdtemp('downloads_yt_')
        yt = VideoMetadates()
        video_info = yt.get_video_info(url_video=video_url)

        if video_info.is_private:
            print("O vídeo é privada. Não é possível baixar vídeos privados")
            return

        video_title = video_info.title
        print(f"Baixando Vídeo: {video_title}")
        uri = self.get_info_video(video_url)
        title_v = self.sanitize_filename(video_title)
        out = os.path.join(output_dir, f"{title_v}.mp4")
        v_path = uri.download_video(title=title_v, overwrite_output=True, output_dir=temp_dir, logs=True)
        uri_a = self.get_info_audio(url_video=video_url)
        title_a = self.sanitize_filename(f"AUDIO_{video_title}")
        print(f"Baixando Audio: {video_title}")
        a_path = uri_a.download_audio(title=title_a, output_dir=temp_dir, overwrite_output=True, logs=True)
        print("Remuxando.........")
        time.sleep(2)
        self.remux(a_path=a_path, v_path=v_path, out=out)
        os.remove(a_path)
        os.remove(v_path)

    def download_video_audio(self, video_url, output_dir):
        os.makedirs(output_dir, exist_ok=True)
        temp_dir = tempfile.mkdtemp('downloads_yt_')
        yt = VideoMetadates()
        video_info = yt.get_video_info(url_video=video_url)
        if video_info.is_private:
            print("O vídeo é privada. Não é possível baixar vídeos privados")
            return
        video_title = video_info.title
        uri_a = self.get_info_audio(url_video=video_url)
        title_a = self.sanitize_filename(f"{video_title}")
        print(f"Baixando Audio: {video_title}")
        uri_a.download_audio(title=title_a, output_dir=temp_dir, overwrite_output=True, logs=True)


def verific(url: str) -> bool:
    """
    Verifica se a URL é de uma playlist do YouTube.
    """
    pattern = r"(https?://)?(www\.)?(youtube\.com|youtu\.?be)/.*[?&]list=([a-zA-Z0-9_-]+)"
    return re.match(pattern, url) is not None


def print_help():
    help_text = """
    yt-downloader vBETA
    ---------------------

    Comandos disponíveis:

    -p, --playlist    URL da playlist do YouTube
    -v, --video       URL do vídeo individual do YouTube
    -a, --audio       Baixar apenas o áudio
    -o, --output      Diretório de saída para os vídeos/áudios baixados (padrão: 'downloads')

    Exemplos:

    1. Baixar todos os vídeos de uma playlist:
       python yt-downloader -p https://www.youtube.com/playlist?list=PLxxxxxx

    2. Baixar um vídeo específico:
       python yt-downloader -v https://www.youtube.com/watch?v=xxxxxx

    3. Baixar apenas o áudio de um vídeo:
       python yt-downloader -v https://www.youtube.com/watch?v=xxxxxx -a

    4. Baixar apenas o áudio de vídeos de uma playlist:
       python yt-downloader -p https://www.youtube.com/playlist?list=PLxxxxxx -a

    5. Especificar um diretório de saída:
       python yt-downloader -v https://www.youtube.com/watch?v=xxxxxx -o meu_diretorio

    """
    print(help_text)


def main():
    # Configurando o argparse para os argumentos fornecidos
    parser = argparse.ArgumentParser(description="Download de vídeos ou playlists do YouTube", add_help=False)

    # Adiciona os argumentos esperados
    parser.add_argument('-h', '--help', action='store_true', help='Mostrar esta mensagem de ajuda')
    parser.add_argument('-p', '--playlist', type=str, help='URL da playlist do YouTube')
    parser.add_argument('-v', '--video', type=str, help='URL do vídeo individual do YouTube')
    parser.add_argument('-a', '--audio', action='store_true', help='Baixar apenas o áudio')
    parser.add_argument('-o', '--output', type=str, default='downloads',
                        help='Diretório de saída para os vídeos/áudios baixados')

    # Parseia os argumentos da linha de comando
    args = parser.parse_args()
    p = Playlists()

    if args.help:
        print_help()
        sys.exit(0)
    if args.playlist:
        # Chamar a função para download da playlist
        if args.audio:
            print(f"Baixando apenas os áudios dos vídeos da playlist: {args.playlist}")
            p.download_playlist_videos(args.playlist, args.output, True)
        else:
            p.download_playlist_videos(args.playlist, args.output)
    elif args.video:
        if args.audio:
            print(f"Baixando apenas o áudio do vídeo: {args.video}")
            # Chamar a função para download do áudio do vídeo
            p.download_video_audio(args.video, args.output)
        else:
            print(f"Baixando o vídeo: {args.video}")
            # Chamar a função para download do vídeo
            p.download_video(args.video, args.output)

    else:
        print("Por favor, forneça uma URL de vídeo (-v) ou de playlist (-p) para baixar.")


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        print("interrompido!")
    except Exception as e:
        pass



from youtube_analyzer.__version__ import __version__, __autor__, __repo__, __source__, __lib__
from setuptools import setup, find_packages

# Lê o conteúdo do README.md
with open('README_PYPI.md', 'r', encoding='utf-8') as file:
    long_description = file.read()

setup(
    name="youtube-analyzer",
    version=__version__,
    description="Analize vídeos do youtube",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author=__autor__,
    author_email="paulocesar0073dev404@gmail.com",
    license="MIT",
    keywords=["youtube","youtube-analyzer","analizer youtube"],
    install_requires=[
        'requests',
        'colorama',
        'beautifulsoup4',
    ],
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    platforms=["any"],
    project_urls={
        "Código Fonte": __source__,
        "lib": __lib__,
        'GitHub': __repo__,
        "Bugs/Melhorias": f"{__repo__}/issues",
        "Documentação": f"{__repo__}/wiki",
    },

    entry_points={
        'console_scripts': [
            'yt-downloader=youtube_analyzer.yt_downloader:main',
        ],
    },
)


from .api import is_valid
from .exeptions import YoutubeAnalyzerExceptions
from .parsers import ParserPlaylist, VideoParser


class VideoMetadates:
    @staticmethod
    def get_video_info(url_video: str) -> VideoParser:
        """
        :param url_video: url do vídeo
        :return: Objeto
        """
        if not is_valid(url=url_video):
            raise YoutubeAnalyzerExceptions("url não é válida!")
        else:
            a = VideoParser(url=url_video)
            return a


class PlaylistMetadates:
    @staticmethod
    def get_playlist_info(playlist_url: str) -> ParserPlaylist:
        """
        ATENÇÃO ESTA AÇÃO VAI DEMORAR DEVIDO AS REQUISIÇÃO PARA OBTER TODOS OS ELEMENTOS DE UMA PLAYLIST,DEPENDENDO
        DO TAMANHO VAI SER DEMORADO... :param playlist_url: url da playlist :return: object
        """
        if not is_valid(url=playlist_url):
            raise YoutubeAnalyzerExceptions("url não é válida!")
        else:
            a = ParserPlaylist(playlist_url=playlist_url)
            return a



__version__ = '0.3.4'
__lib_name__ = 'youtube_analyzer'
__autor__ = 'PauloCesar-dev404'
__repo__ = f'https://github.com/PauloCesar-dev404/{__lib_name__}'
__lib__ = f'https://raw.githubusercontent.com/PauloCesar-dev404/{__lib_name__}/main/{__lib_name__}-{__version__}-py3-none-any.whl'
__source__ = f'https://raw.githubusercontent.com/PauloCesar-dev404/{__lib_name__}/main/{__lib_name__}-{__version__}.tar.gz'

